from __future__ import annotations

from typing import Sequence

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..Animation.AnimationOptions import AnimationOptions
from ..Animation.AVIOptions import AVIOptions
from ..Animation.ImageAnimation import ImageAnimation
from ..Animation.ImageAnimationOptions import ImageAnimationOptions
from ..Animation.Movie import Movie
from ..Animation.QuickTimeOptions import QuickTimeOptions
from ..Canvas.Canvas import Canvas
from ..Canvas.DrawingArea import DrawingArea
from ..Canvas.Viewport import Viewport
from ..CustomKernel.RepositorySupport import RepositorySupport
from ..DisplayGroup.DisplayGroup import DisplayGroup
from ..DisplayOptions.GraphicsInfo import GraphicsInfo
from ..DisplayOptions.GraphicsOptions import GraphicsOptions
from ..DisplayOptions.LightOptions import LightOptions
from ..DisplayOptions.ViewportAnnotationOptions import ViewportAnnotationOptions
from ..FieldReport.FieldReportOptions import FieldReportOptions
from ..FieldReport.FreeBodyReportOptions import FreeBodyReportOptions
from ..Job.Queue import Queue
from ..Mesh.MesherOptions import MesherOptions
from ..Odb.Odb import Odb
from ..Odb.ScratchOdb import ScratchOdb
from ..OdbDisplay.DefaultOdbDisplay import DefaultOdbDisplay
from ..OdbDisplay.ViewerOptions import ViewerOptions
from ..PathAndProbe.CurrentProbeValues import CurrentProbeValues
from ..PathAndProbe.FreeBody import FreeBody
from ..PathAndProbe.NodeQuery import NodeQuery
from ..PathAndProbe.Path import Path
from ..PathAndProbe.ProbeOptions import ProbeOptions
from ..PathAndProbe.ProbeReport import ProbeReport
from ..PathAndProbe.SelectedProbeValues import SelectedProbeValues
from ..PathAndProbe.Spectrum import Spectrum
from ..PathAndProbe.Stream import Stream
from ..PlotOptions.MdbData import MdbData
from ..PlotOptions.OdbData import OdbData
from ..PredefinedField.TiffOptions import TiffOptions
from ..Print.EpsOptions import EpsOptions
from ..Print.PageSetupOptions import PageSetupOptions
from ..Print.PngOptions import PngOptions
from ..Print.PrintOptions import PrintOptions
from ..Print.PsOptions import PsOptions
from ..Print.SvgOptions import SvgOptions
from ..Sketcher.ConstrainedSketchOptions.ConstrainedSketcherOptions import (
    ConstrainedSketcherOptions,
)
from ..UtilityAndView.abaqusConstants import AVI, OFF, PNG, Boolean
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from ..UtilityAndView.View import View
from ..XY.Chart import Chart
from ..XY.DefaultChartOptions import DefaultChartOptions
from ..XY.DefaultPlot import DefaultPlot
from ..XY.XYCurve import XYCurve
from ..XY.XYData import XYData
from ..XY.XYPlot import XYPlot
from ..XY.XYReportOptions import XYReportOptions
from .AutoColors import AutoColors
from .Color import Color
from .Drawing import Drawing
from .Image import Image
from .JournalOptions import JournalOptions
from .MemoryReductionOptions import MemoryReductionOptions
from .NetworkDatabaseConnector import NetworkDatabaseConnector


@abaqus_class_doc
class SessionBase:
    """The Session object has no constructor. Abaqus creates the **session** member when a session is started.

    .. note::
        This object can be accessed by::

            session
    """

    #: A Boolean specifying whether an Abaqus interactive session is running.
    attachedToGui: Boolean = OFF

    #: A Boolean specifying whether Abaqus is executing a replay file.
    replayInProgress: Boolean = OFF

    #: A Float specifying the memory usage value for the Abaqus/CAE kernel process in
    #: megabytes.
    kernelMemoryFootprint: float | None = None

    #: A Float specifying the maximum value for the memory usage for the Abaqus/CAE kernel
    #: process in megabytes.
    kernelMemoryMaxFootprint: float | None = None

    #: A Float specifying the limit for the memory use for the Abaqus/CAE kernel process in
    #: megabytes.
    kernelMemoryLimit: float | None = None

    #: A repository of Color objects.
    colors: dict[str, Color] = {}

    #: A JournalOptions object specifying how to record selection of geometry in the journal
    #: and replay files.
    journalOptions: JournalOptions = JournalOptions()

    #: A MemoryReductionOptions object specifying options for running in reduced memory mode.
    memoryReductionOptions: MemoryReductionOptions = MemoryReductionOptions()

    #: A NodeQuery object specifying nodes and their coordinates in a path.
    nodeQuery: NodeQuery = NodeQuery()

    #: A ConstrainedSketcherOptions object specifying common options for all sketches.
    sketcherOptions: ConstrainedSketcherOptions = ConstrainedSketcherOptions()

    #: A ViewerOptions object.
    viewerOptions: ViewerOptions = ViewerOptions()

    #: An AnimationOptions object.
    animationOptions: AnimationOptions = AnimationOptions()

    #: An AVIOptions object.
    aviOptions: AVIOptions = AVIOptions()

    #: An ImageAnimationOptions object.
    imageAnimationOptions: ImageAnimationOptions = ImageAnimationOptions()

    #: An ImageAnimation object.
    imageAnimation: ImageAnimation = ImageAnimation("img", AVI)

    #: A QuickTimeOptions object.
    quickTimeOptions: QuickTimeOptions = QuickTimeOptions()

    #: A repository of Viewport objects.
    viewports: dict[str, Viewport] = {}

    #: A RepositorySupport object.
    customData: RepositorySupport = RepositorySupport()

    #: A FieldReportOptions object.
    defaultFieldReportOptions: FieldReportOptions = FieldReportOptions()

    #: A FreeBodyReportOptions object.
    defaultFreeBodyReportOptions: FreeBodyReportOptions = FreeBodyReportOptions()

    #: A FieldReportOptions object.
    fieldReportOptions: FieldReportOptions = FieldReportOptions()

    #: A FreeBodyReportOptions object.
    freeBodyReportOptions: FreeBodyReportOptions = FreeBodyReportOptions()

    #: A repository of Odb objects.
    odbs: dict[str, Odb] = {}

    #: A repository of ScratchOdb objects.
    scratchOdbs: dict[str, ScratchOdb] = {}

    #: A DefaultOdbDisplay object.
    defaultOdbDisplay: DefaultOdbDisplay = DefaultOdbDisplay()

    #: A DefaultPlot object.
    defaultPlot: DefaultPlot = DefaultPlot()

    #: A DefaultChartOptions object.
    defaultChartOptions: DefaultChartOptions = DefaultChartOptions()

    #: A repository of OdbData objects.
    odbData: dict[str, OdbData] = {}

    #: A repository of MdbData objects.
    mdbData: dict[str, MdbData] = {}

    #: A repository of Path objects.
    paths: dict[str, Path] = {}

    #: A repository of FreeBody objects.
    freeBodies: dict[str, FreeBody] = {}

    #: A repository of Stream objects.
    streams: dict[str, Stream] = {}

    #: A repository of Spectrum objects.
    spectrums: dict[str, Spectrum] = {}

    #: A CurrentProbeValues object.
    currentProbeValues: CurrentProbeValues = CurrentProbeValues()

    #: A ProbeOptions object.
    defaultProbeOptions: ProbeOptions = ProbeOptions()

    #: A ProbeOptions object.
    probeOptions: ProbeOptions = ProbeOptions()

    #: A ProbeReport object.
    probeReport: ProbeReport = ProbeReport()

    #: A ProbeReport object.
    defaultProbeReport: ProbeReport = ProbeReport()

    #: A SelectedProbeValues object.
    selectedProbeValues: SelectedProbeValues = SelectedProbeValues()

    #: A PrintOptions object.
    printOptions: PrintOptions = PrintOptions()

    #: An EpsOptions object.
    epsOptions: EpsOptions = EpsOptions()

    #: A PageSetupOptions object.
    pageSetupOptions: PageSetupOptions = PageSetupOptions()

    #: A PngOptions object.
    pngOptions: PngOptions = PngOptions()

    #: A PsOptions object.
    psOptions: PsOptions = PsOptions()

    #: A SvgOptions object.
    svgOptions: SvgOptions = SvgOptions()

    #: A TiffOptions object.
    tiffOptions: TiffOptions = TiffOptions()

    #: An AutoColors object specifying the color palette to be used for color coding.
    autoColors: AutoColors = AutoColors()

    #: An AutoColors object specifying the color palette to be used forXYCurve objects.
    xyColors: AutoColors = AutoColors()

    #: A repository of XYData objects.
    xyDataObjects: dict[str, XYData] = {}

    #: A repository of XYCurve objects.
    curves: dict[str, XYCurve] = {}

    #: A repository of XYPlot objects.
    xyPlots: dict[str, XYPlot] = {}

    #: A repository of Chart objects.
    charts: dict[str, Chart] = {}

    #: An XYReportOptions object.
    defaultXYReportOptions: XYReportOptions = XYReportOptions()

    #: An XYReportOptions object.
    xyReportOptions: XYReportOptions = XYReportOptions()

    #: A repository of View objects.
    views: dict[str, View] = {}

    #: A repository of NetworkDatabaseConnector objects.
    networkDatabaseConnectors: dict[str, NetworkDatabaseConnector] = {}

    #: A repository of DisplayGroup objects.
    displayGroups: dict[str, DisplayGroup] = {}

    #: A GraphicsInfo object.
    graphicsInfo: GraphicsInfo = GraphicsInfo()

    #: A GraphicsOptions object.
    defaultGraphicsOptions: GraphicsOptions = GraphicsOptions()

    #: A GraphicsOptions object.
    graphicsOptions: GraphicsOptions = GraphicsOptions()

    #: A ViewportAnnotationOptions object.
    defaultViewportAnnotationOptions: ViewportAnnotationOptions = ViewportAnnotationOptions()

    #: A repository of Queue objects.
    queues: dict[str, Queue] = {}

    #: A String specifying the name of the current viewport.
    currentViewportName: str = ""

    #: A Dictionary object specifying the viewports and their associated models. The Dictionary
    #: key specifies the viewport name. The Dictionary value is a Dictionary specifying the
    #: model name.
    sessionState: dict | None = None

    #: A repository of Image objects.
    images: dict[str, Image] = {}

    #: A repository of Movie objects.
    movies: dict[str, Movie] = {}

    #: A LightOptions object.
    defaultLightOptions: LightOptions = LightOptions()

    #: A DrawingArea object.
    drawingArea: DrawingArea = DrawingArea()

    #: A MesherOptions object specifying how to control default settings in the Mesh module.
    defaultMesherOptions: MesherOptions = MesherOptions()

    #: A repository of Drawing objects.
    drawings: dict[str, Drawing] = {}

    @abaqus_method_doc
    def setValues(self, kernelMemoryLimit: float | None = None):
        """This method modifies the Session object.

        Parameters
        ----------
        kernelMemoryLimit
            A Float specifying the memory limit value for the Abaqus/CAE kernel process in
            megabytes. If the limit is exceeded, Abaqus/CAE displays an error message.The default
            memory limit value for Windows 32-bit systems if not set is 1800 MB. Increasing the
            memory limit is not recommended unless you are using a Windows 32-bit system with the
            boot option `/3GB /userva = SizeInMBytes` to extend the amount of memory available for
            Abaqus/CAE. In this case the limit can be changed to 2800 MB.If the kernel memory size
            reaches the **abq_ker_memory** value or the virtual memory limit of the machine, the
            following message will be displayed:

            `Operation did not complete due to a memory allocation failure.`

            For optimal performance, the memory limit should be set to a value
            less than the physical amount of memory on the machine.The minimum setting allowed is
            256 MB.
        """
        ...

    @abaqus_method_doc
    def enableCADConnection(self, CADName: str, portNum: int | None = None):
        """This method enables the Abaqus/CAE listening port for the specified **CAD** system.

        Parameters
        ----------
        CADName
            A String specifying the CAD system. Available options are Pro/ENGINEER, CATIA V5, CATIA
            V6, NX and SolidWorks.
        portNum
            An Integer specifying the port number to be used by the CAD system to communicate with
            Abaqus/CAE. If unspecified, attempts will be made to identify an open port. The default
            ports used are as follow: Pro/E : 49178CATIA V5 : 49179SolidWorks : 49180NX : 49181CATIA
            V6 : 49182

        Returns
        -------
        int
            The connection port number used for the CAD connection.
        """
        ...

    @abaqus_method_doc
    def isCADConnectionEnabled(self):
        """This method checks the status of CAD Connection.

        Returns
        -------
        Boolean
            A Boolean value of True if the CAD connection enabled and False if the CAD connection
            disabled.
        """
        ...

    @abaqus_method_doc
    def disableCADConnection(self, CADName: str):
        """This method disables an associative import CAD connection that was enabled.

        Parameters
        ----------
        CADName
            A String specifying the CAD system for which associative import will be disabled.
            Available options are Pro/ENGINEER, CATIA V5, and CATIA V6, NX and SolidWorks.
        """
        ...

    @abaqus_method_doc
    def enableParameterUpdate(self, CADName: str, CADVersion: str, CADPort: int | None = None):
        """This method enables parameter updates for ProE and NX by establishing a connection with the listening
        port previously setup by the CAD application.

        Parameters
        ----------
        CADName
            A String specifying the CAD system for which parameter update will be enabled. Available
            options are Pro/ENGINEER and NX.
        CADVersion
            A String specifying the CAD system version. Allowable options take the form of the
            specific CAD system plus a version string. Examples for Pro/ENGINEER are "Wildfire5" and
            "Creo4." An NX example is "NX11."
        CADPort
            An Integer specifying the port number to be used by Abaqus/CAE to communicate with the
            CAD system. If unspecified, attempts will be made to identify an open port. This port
            number is not the same as the **portNum** used by the associative import interface. The
            default CAD listening ports are as follow:ProE : 3344NX : 3344
        """
        ...

    @abaqus_method_doc
    def setCADPortNumber(self, CADName: str, Port: str):
        """This method enables parameter updates for CATIA V5 and CATIA V6 by establishing a connection with the
        listening port previously setup by the CAD application. This port number is used to send the parameter
        information to the CAD system.

        Parameters
        ----------
        CADName
            A String specifying the CAD system for which the port number will be saved. The
            available options are 'CATIA V5' and ' CATIA V6'.
        Port
            An integer specifying the port number to be used by Abaqus/CAE to send the modified
            parameters to the CAD system.
        """
        ...

    @abaqus_method_doc
    def updateCADParameters(
        self,
        modelName: str,
        CADName: str,
        parameterFile: str,
        CADPartFile: str,
        CADPartName: str = "",
    ):
        """This method updates the parameters for the specified model using the specified parameter file.

        Parameters
        ----------
        modelName
            A String specifying the model name to update.
        CADName
            A String specifying the CAD system for which Abaqus updates the parameters. The
            available options are 'Pro/ENGINEER', 'CATIA V5' and 'CATIA V6'.
        parameterFile
            A parameter file containing the parameters that were exposed in the CAD system using the
            ``ABQ_`` prefix.
        CADPartFile
            A file name specifying the CAD part file for which parameter update is triggered.For
            **CADName** = 'CATIA V5' or 'CATIA V6', you can specify either products or parts using this
            argument. If you specify a product, Abaqus updates all of the parts associated with that
            product.For **CADName** = 'Pro/ENGINEER', this argument is optional, and you can specify
            update for parts only. However, a single file can be associated with multiple parts in
            the case of family tables. In this case, Abaqus updates all listed parts.
        CADPartName
            An String specifying the part name to update. This part name should match the part name
            in the originating CAD system.If you specify neither **CADPartFile** nor **CADPartName**
            during an update in which you specified **CADName** = 'Pro/ENGINEER', Abaqus updates all of
            the parts in the specified file.
        """
        ...

    @abaqus_method_doc
    def disableParameterUpdate(self, CADName: str):
        """This method disables an associative CAD connection using parameters.

        Parameters
        ----------
        CADName
            A String specifying the CAD system for which the parameter update will be disabled.
            Available option is Pro/ENGINEER.
        """
        ...

    @abaqus_method_doc
    def printToFile(
        self,
        fileName: str,
        format: Literal[C.PS, C.EPS, C.PNG, C.TIFF, C.SVG] = PNG,
        canvasObjects: Sequence[Canvas] = (),
        compression: Boolean = OFF,
    ):
        """This method prints canvas objects to a file using the attributes stored in the PrintOptions object
        and the appropriate format options object.

        Parameters
        ----------
        fileName
            A String specifying the file to which the image is to be written. If no file extension
            is supplied, an extension is added based on the selected image format (.ps, .eps, .png,
            .tif, .svg, or .svgz).
        format
            A SymbolicConstant specifying the image format. Possible values are PNG, SVG, TIFF, PS,
            and EPS. The default value is PNG.
        canvasObjects
            A sequence of canvas objects (viewports, text strings, or arrows) to print. The default
            is to print all canvas objects.
        compression
            A Boolean specifying the format for an SVG file. It is valid to use this argument
            only when **format** is SVG. Possible values are False (Uncompressed) and True (Compressed).

            .. versionchanged:: 2022
                It is "only" valid to use this argument ...
        """
        ...

    @abaqus_method_doc
    def printToPrinter(
        self,
        printCommand: str = "",
        numCopies: int = 1,
        canvasObjects: Sequence[Canvas] = (),
    ):
        """This method prints canvas objects to a Windows printer or to a PostScript printer. The attributes
        used for printing to a Windows printer are stored in the PrintOptions object and the PageSetupOptions
        object; the attributes used for printing to a PostScript printer are stored in the PrintOptions object
        and the PsOptions object.

        Parameters
        ----------
        printCommand
            A String specifying the operating system command or printer name to issue for printing
            to the printer. The default value is “lpr” or the value specified by the printOptions
            method. If you create a script to print directly to a Windows printer, the
            **printCommand** must take the following
            form::

                session.printToPrinter.setValues(
                    printCommand='PRINTER[number of characters in name]:printername PROPERTIES[number of characters in properties]:document properties'
                )

            The ``PROPERTIES`` is a list of characters that represents the
            printing preferences for the selected Windows printer. The properties are not required
            in a script; the printed output will use the current settings for the selected printer.
            You can use `'PRINTER[7]: DEFAULT'` to specify the default Windows printer.
        numCopies
            An Int specifying the number of copies to print. Possible values are 1 ≤ **numCopies** ≤
            100. The default value is 1.
        canvasObjects
            A sequence of canvas objects (viewports, text strings, or arrows) to print. The default
            is to print all canvas objects.

        Raises
        ------
        SystemError
            invalid print command, If **printCommand** is invalid.
        SystemError
            print command failed, If the print command fails.
        RangeError: numCopies must be in the range 1 <= value <= 100
            If **numCopies** is out of range.
        TypeError
            keyword error on compression, If **compression** is specified when **format** is not SVG.
        """
        ...

    @abaqus_method_doc
    def saveOptions(self, directory: Literal[C.HOME, C.CURRENT]):
        """This method saves your customized display settings.

        Parameters
        ----------
        directory
            A SymbolicConstant specifying the directory in which Abaqus saves the file that will be
            used to restore your customized settings (abaqus_2021.gpr). Possible values are HOME and
            CURRENT.
        """
        ...

    @abaqus_method_doc
    def writeVrmlFile(self, fileName: str, format: Boolean = OFF, canvasObjects: Sequence[Canvas] = ()):
        """This method exports the current viewport objects to a file.

        Parameters
        ----------
        fileName
            A String specifying the file to which the graphics data is to be written. If no file
            extension is supplied, an extension is added based on the selected format (.wrl, .wrz).
        format
            A Boolean specifying the format. Possible values are False (Uncompressed) and True
            (Compressed).
        canvasObjects
            A sequence of canvas objects (viewports, text strings, or arrows) to export.
        """
        ...

    @abaqus_method_doc
    def write3DXMLFile(self, fileName: str, format: Boolean = OFF, canvasObjects: Sequence[Canvas] = ()):
        """This method exports the current viewport objects to a file.

        Parameters
        ----------
        fileName
            A String specifying the file to which the graphics data is to be written. If no file
            extension is supplied, (.3dxml) will be added.
        format
            A Boolean specifying the format. Possible values are False (Uncompressed) and True
            (Compressed).
        canvasObjects
            A sequence of canvas objects to export.
        """
        ...

    @abaqus_method_doc
    def writeOBJFile(self, fileName: str, canvasObjects: Sequence[Canvas] = ()):
        """This method exports the current viewport objects to a file.

        Parameters
        ----------
        fileName
            A String specifying the file to which the graphics data is to be written. If no file
            extension is supplied, (.obj) will be added.
        canvasObjects
            A sequence of canvas objects to export.
        """
        ...

    # The following method was originally in the ``OdbCommands`` page documentation
    # But it accessed only by ``session`` object.
    @abaqus_method_doc
    def openOdb(self, name: str, path: str = "", readOnly: Boolean = OFF) -> Odb:
        """This method opens an existing output database (`.odb`) file and creates a new Odb object. This method
        is accessed only via the session object inside Abaqus/CAE and adds the new Odb object to the
        session.odbs repository. This method allows you to open multiple output databases at the same time and
        to use the repository key to specify a particular output database. For example::

            import visualization
            session.openOdb(name='myOdb', path='stress.odb', readOnly=True)

        Parameters
        ----------
        name
            A String specifying the repository key. If the ``name`` is not the same as the ``path`` to the
            output database (`.odb`) file, the ``path`` must be specified as well. Additionally, to
            support backwards compatibility of the interface, if the ``name`` parameter is omitted,
            the ``path`` and ``name`` will be presumed to be the same.
        path
            A String specifying the path to an existing output database (`.odb`) file.
        readOnly
            A Boolean specifying whether the file will permit only read access or both read and
            write access. The initial value is TRUE when the output database file is opened from
            Abaqus/CAE, indicating that only read access will be permitted.

        Returns
        -------
        Odb
            An Odb object.

        Raises
        ------
        OdbError
            The database is from a previous release of Abaqus, If the output database was generated by a previous release of Abaqus and needs
            upgrading, Run  `abaqus upgrade -job <newFilename> -odb <oldFileName>` to upgrade it.
        OdbError
            Abaqus installation must be upgraded before this output database can be opened, If the output database was generated by a newer release of Abaqus, and the
            installation of Abaqus needs upgrading.
        AbaqusError: Cannot open file <filename>
            If the file is not a valid database.
        """
        self.odbs[name] = odb = Odb(name)
        return odb



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc

from ..UtilityAndView.abaqusConstants import SymbolicConstant
from .OdbSet import OdbSet


@abaqus_class_doc
class BeamOrientation:
    """The BeamOrientation object represents the direction of the first beam section axis n1n1. Specifying the
    beam orientation using an additional node in the element connectivity list is not supported.

    .. note::
        This object can be accessed by::

            import odbAccess
            session.odbs[name].parts[name].beamOrientations[i]
            session.odbs[name].rootAssembly.instances[name].beamOrientations[i]
            session.odbs[name].steps[name].frames[i].fieldOutputs[name].values[i].instance.beamOrientations[i]
    """

    #: A SymbolicConstant specifying the orientation assignment method. Possible values are
    #: N1_COSINES, CSYS, and VECT.
    method: SymbolicConstant

    #: An OdbSet object specifying a region for which the beam orientation is defined.
    region: OdbSet = OdbSet("set", ())

    #: A tuple of Floats specifying direction cosines of the n1-direction of the beam
    #: cross-section.
    vector: float | None = None



from __future__ import annotations

from typing import Union

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import (
    AUTOCOMPUTE,
    AUTOMATIC,
    LINEAR,
    ON,
    Boolean,
    SymbolicConstant,
)
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .QuantityType import QuantityType
from .XYCurveArray import XYCurveArray


@abaqus_class_doc
class AxisData:
    """The AxisData object is used to store the data attributes of axes. An AxisData object is automatically
    created when creating an Axis object.

    .. note::
        This object can be accessed by::

            import visualization
            session.charts[name].axes1[i].axisData
            session.charts[name].axes2[i].axisData
            session.defaultChartOptions.defaultAxis1Options.axisData
            session.defaultChartOptions.defaultAxis2Options.axisData
            session.xyPlots[name].charts[name].axes1[i].axisData
            session.xyPlots[name].charts[name].axes2[i].axisData
    """

    #: A Float specifying the reference value for decibel computation. The default value is
    #: 1.0.
    dbReference: float = 1

    #: A SymbolicConstant specifying the direction of the axis. Possible values are ABSCISSA
    #: and ORDINATE.
    direction: SymbolicConstant

    #: A SymbolicConstant specifying how tick labels are formatted. Possible values are
    #: AUTOMATIC, DECIMAL, SCIENTIFIC, and ENGINEERING. The default value is AUTOMATIC.
    labelFormat: SymbolicConstant = AUTOMATIC

    #: An Int specifying the number of significant digits displayed for the labels. Possible
    #: values are 1 to 7. The default value is 2.
    labelNumDigits: int = 2

    #: A Boolean specifying whether or not to use the automatically computed maximum value for
    #: the axis. The default value is ON.
    maxAutoCompute: Boolean = ON

    #: A Float specifying the maximum value when **maxAutoCompute** is true.
    maxAutoValue: float | None = None

    #: A Float specifying the maximum value when **maxAutoCompute** is false. By default,
    #: **maxValue** is set to **maxAutoValue**.
    maxValue: float | None = None

    #: A Float specifying the current maximum value displayed for this axis. This value is
    #: different from **maxAutoValue** or **maxValue** when the axis is being transformed (zoom or
    #: pan).
    maxShownValue: float | None = None

    #: A Boolean specifying whether or not to use the automatically computed minimum value for
    #: the axis. The default value is ON.
    minAutoCompute: Boolean = ON

    #: A Float specifying the minimum value when **minAutoCompute** is true.
    minAutoValue: float | None = None

    #: A Float specifying the minimum value when **minAutoCompute** is false. By default,
    #: **minValue** is set to **minAutoValue**.
    minValue: float | None = None

    #: A Float specifying the current minimum value displayed for this axis. This value is
    #: different from **minAutoValue** or **minValue** when the axis is being transformed (zoom or
    #: pan).
    minShownValue: float | None = None

    #: An Int specifying the number the number of minor tick marks between major ticks.
    #: Possible values are 0 ≤ **minorTickCount** ≤ 20. When the **scale** is set to LOG, the
    #: minorTickCount is interpreted as the number of ticks per decade and limited to 0, 1, 4,
    #: 8, and 17. The default value is 1.
    minorTickCount: int = 1

    #: A SymbolicConstant specifying the type of scale to use for the axis. Possible values
    #: are:LINEAR, specifying tickmarks and labels are linearly distributed.LOG, specifying
    #: tickmarks and labels are logarithmically distributed.DB, specifying tickmarks and labels
    #: are distributed on a decibel scale.DB2, specifying tickmarks and labels are distributed
    #: on a 2*decibel scale.The default value is LINEAR.
    scale: SymbolicConstant = LINEAR

    #: A SymbolicConstant specifying the type of scale to use for the axis. Possible values
    #: are:AUTOCOMPUTE, specifying tickmarks and labels are automatically computed.INCREMENT,
    #: specifying tickmarks and labels are defined by a given increment.TOTAL_NUMBER,
    #: specifying tickmarks and labels are defined by the total number of ticks.The default
    #: value is AUTOCOMPUTE.
    tickMode: SymbolicConstant = AUTOCOMPUTE

    #: An Int specifying the number of major tick marks on the axis when **tickMode**
    #: =TOTAL_NUMBER. Possible values are 0 ≤ **tickCount** ≤ 30. The default value is computed
    #: based on the range of the axis. When the **scale** is set to LOG, the tickCount is
    #: interpreted as the number of ticks per decade and acceptable values are 1, 4, 8, and 17.
    tickCount: int | None = None

    #: An Int specifying the number of major ticks effectively shown. This value takes zoom,
    #: pan and rounding into account.
    tickCountShown: int | None = None

    #: A Float specifying the increment of the major tick marks on the axis when **tickMode** =
    #: INCREMENT. Valid values are 0 < **tickIncrement**. The default value is computed based on
    #: the results of the automatic method and the range being plotted. When the **scale** is set
    #: to LOG, the tickIncrement is interpreted as a value per decade and should be between
    #: 0.05 and 1.
    tickIncrement: float | None = None

    #: A Float specifying the shown tick increment of the major ticks. This value takes
    #: zoom/pan into account.
    tickIncrementShown: float | None = None

    #: A Boolean specifying whether the title to use for the axis title is system defined or
    #: user defined. The default value is ON.
    useSystemTitle: Boolean = ON

    #: An XYCurveArray object specifying a read-only sequence of Curve objects associated to
    #: this axis.
    curves: XYCurveArray = []

    #: A QuantityType object specifying the quantity type: i.e. the physical dimension and
    #: associated label of the data represented by this axis.
    quantityType: QuantityType = QuantityType()

    #: A tuple of Floats specifying the read-only major tick values shown.
    tickValues: float | None = None

    #: A tuple of Strings specifying the read-only major tick labels shown.
    tickLabels: tuple = ()

    #: A String specifying the system title. The system title is based on the **quantityType** of
    #: the axis and associated curves.
    systemTitle: str = ""

    #: A String specifying the title of the axis. By default, the title is set to the
    #: **systemTitle**.
    title: str = ""

    @abaqus_method_doc
    def setValues(
        self,
        axisData: Union["AxisData", None] = None,
        labelFormat: Literal[C.SCIENTIFIC, C.AUTOMATIC, C.ENGINEERING, C.DECIMAL] = AUTOMATIC,
        labelNumDigits: int = 2,
        scale: Literal[C.LINEAR] = LINEAR,
        dbReference: float = 1,
        minAutoCompute: Boolean = ON,
        minValue: float | None = None,
        maxAutoCompute: Boolean = ON,
        maxValue: float | None = None,
        tickMode: Literal[C.AUTOCOMPUTE] = AUTOCOMPUTE,
        tickIncrement: float | None = None,
        tickCount: int | None = None,
        minorTickCount: int = 1,
        title: str = "",
        useSystemTitle: Boolean = ON,
    ):
        """This method modifies the AxisData object.

        Parameters
        ----------
        axisData
            An AxisData object from which attributes are to be copied.
        labelFormat
            A SymbolicConstant specifying how tick labels are formatted. Possible values are
            AUTOMATIC, DECIMAL, SCIENTIFIC, and ENGINEERING. The default value is AUTOMATIC.
        labelNumDigits
            An Int specifying the number of significant digits displayed for the labels. Possible
            values are 1 to 7. The default value is 2.
        scale
            A SymbolicConstant specifying the type of scale to use for the axis. Possible values
            are:LINEAR, specifying tickmarks and labels are linearly distributed.LOG, specifying
            tickmarks and labels are logarithmically distributed.DB, specifying tickmarks and labels
            are distributed on a decibel scale.DB2, specifying tickmarks and labels are distributed
            on a 2*decibel scale.The default value is LINEAR.
        dbReference
            A Float specifying the reference value for decibel computation. The default value is
            1.0.
        minAutoCompute
            A Boolean specifying whether or not to use the automatically computed minimum value for
            the axis. The default value is ON.
        minValue
            A Float specifying the minimum value when **minAutoCompute** is false. By default,
            **minValue** is set to **minAutoValue**.
        maxAutoCompute
            A Boolean specifying whether or not to use the automatically computed maximum value for
            the axis. The default value is ON.
        maxValue
            A Float specifying the maximum value when **maxAutoCompute** is false. By default,
            **maxValue** is set to **maxAutoValue**.
        tickMode
            A SymbolicConstant specifying the type of scale to use for the axis. Possible values
            are:AUTOCOMPUTE, specifying tickmarks and labels are automatically computed.INCREMENT,
            specifying tickmarks and labels are defined by a given increment.TOTAL_NUMBER,
            specifying tickmarks and labels are defined by the total number of ticks.The default
            value is AUTOCOMPUTE.
        tickIncrement
            A Float specifying the increment of the major tick marks on the axis when **tickMode** =
            INCREMENT. Valid values are 0 < **tickIncrement**. The default value is computed based on
            the results of the automatic method and the range being plotted. When the **scale** is set
            to LOG, the tickIncrement is interpreted as a value per decade and should be between
            0.05 and 1.
        tickCount
            An Int specifying the number of major tick marks on the axis when **tickMode**
            =TOTAL_NUMBER. Possible values are 0 ≤ **tickCount** ≤ 30. The default value is computed
            based on the range of the axis. When the **scale** is set to LOG, the tickCount is
            interpreted as the number of ticks per decade and acceptable values are 1, 4, 8, and 17.
        minorTickCount
            An Int specifying the number the number of minor tick marks between major ticks.
            Possible values are 0 ≤ **minorTickCount** ≤ 20. When the **scale** is set to LOG, the
            minorTickCount is interpreted as the number of ticks per decade and limited to 0, 1, 4,
            8, and 17. The default value is 1.
        title
            A String specifying the title of the axis. By default, the title is set to the
            **systemTitle**.
        useSystemTitle
            A Boolean specifying whether the title to use for the axis title is system defined or
            user defined. The default value is ON.

        Raises
        ------
        RangeError
        """
        ...



from __future__ import annotations

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..Datum.DatumCsys import DatumCsys
from ..Region.Region import Region
from ..UtilityAndView.abaqusConstants import SymbolicConstant
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class AdaptiveMeshConstraint:
    """The AdaptiveMeshConstraint object is the abstract base type for other Arbitrary Lagrangian Eularian (ALE)
    style AdaptiveMeshConstraint objects. The AdaptiveMeshConstraint object has no explicit constructor. The
    methods and members of the AdaptiveMeshConstraint object are common to all objects derived from the
    AdaptiveMeshConstraint object.

    .. note::
        This object can be accessed by::

            import step
            mdb.models[name].adaptiveMeshConstraints[name]
    """

    #: A String specifying the adaptive mesh constraint repository key.
    name: str = ""

    #: A SymbolicConstant specifying the category of the adaptive mesh constraint. Possible
    #: values are MECHANICAL and THERMAL.
    category: SymbolicConstant

    #: A Region object specifying the region to which the adaptive mesh constraint is applied.
    region: Region = Region()

    #: None or a DatumCsys object specifying the local coordinate system of the adaptive mesh
    #: constraint's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
    #: in the global coordinate system. The default value is None.
    localCsys: DatumCsys | None = DatumCsys()

    @abaqus_method_doc
    def __init__(
        self,
        name: str,
        category: Literal[C.THERMAL, C.MECHANICAL],
        region: Region,
        localCsys: DatumCsys | None = None,
    ):
        """The AdaptiveMeshConstraint object is the abstract base type for other Arbitrary Lagrangian Eularian
        (ALE) style AdaptiveMeshConstraint objects. The AdaptiveMeshConstraint object has no explicit
        constructor. The methods and members of the AdaptiveMeshConstraint object are common to all objects
        derived from the AdaptiveMeshConstraint object.

        .. note::
            This function can be accessed by::

                mdb.models[name].AdaptiveMeshConstraint

        Parameters
        ----------
        name
            A String specifying the adaptive mesh constraint repository key.
        category
            A SymbolicConstant specifying the category of the adaptive mesh constraint. Possible values are MECHANICAL and THERMAL.
        region
            A Region object specifying the region to which the adaptive mesh constraint is applied.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the adaptive mesh  constraint's
            degrees of freedom. If **localCsys** = None, the degrees of freedom are defined  in the global coordinate
            system. The default value is None.
        """
        ...

    @abaqus_method_doc
    def deactivate(self, stepName: str):
        """This method deactivates the adaptive mesh constraint in the specified step and all subsequent steps.

        Parameters
        ----------
        stepName
            A String specifying the name of the step in which the adaptive mesh constraint is
            deactivated.

        Raises
        ------
        TextError
        """
        ...

    @abaqus_method_doc
    def move(self, fromStepName: str, toStepName: str):
        """This method moves the adaptive mesh constraint state from one step to a different step.

        Parameters
        ----------
        fromStepName
            A String specifying the name of the step from which the adaptive mesh constraint state
            is moved.
        toStepName
            A String specifying the name of the step to which the adaptive mesh constraint state is
            moved.

        Raises
        ------
        TextError
        """
        ...

    @abaqus_method_doc
    def reset(self, stepName: str):
        """This method resets the adaptive mesh constraint state of the specified step to the state of the
        previous analysis step.

        Parameters
        ----------
        stepName
            A String specifying the name of the step in which the adaptive mesh constraint state is
            reset.

        Raises
        ------
        TextError
        """
        ...

    @abaqus_method_doc
    def resume(self):
        """This method resumes the adaptive mesh constraint that was previously suppressed."""
        ...

    @abaqus_method_doc
    def suppress(self):
        """This method suppresses the adaptive mesh constraint."""
        ...

    @abaqus_method_doc
    def delete(self, indices: tuple):
        """This method allows you to delete existing adaptive mesh constraints.

        Parameters
        ----------
        indices
            A sequence of Ints specifying the index of each adaptive mesh constraint to delete.
        """
        ...



from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import SymbolicConstant
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class MonitorMgr:
    """An instance of the MonitorMgr object is created when you import the abaqus module. No other instance of
    the MonitorMgr object is required. (This MonitorMgr object is not to be confused with the degree of freedom
    (DOF) monitor that is constructed from the Step object.)

    .. note::
        This object can be accessed by::

            monitorManager
    """

    @abaqus_method_doc
    def addMessageCallback(
        self,
        jobName: str,
        messageType: Literal[
            C.HEALER_TYPE,
            C.MONITOR_DATA,
            C.JOB_COMPLETED,
            C.JOB_SUBMITTED,
            C.SIMULATION_SUBMITTED,
            C.STEP,
            C.WARNING,
            C.JOB_ABORTED,
            C.SIMULATION_INTERRUPTED,
            C.COMPLETED,
            C.ITERATION,
            C.STATUS,
            C.SIMULATION_COMPLETED,
            C.ODB_FRAME,
            C.ABORTED,
            C.JOB_INTERRUPTED,
            C.ANY_MESSAGE_TYPE,
            C.END_STEP,
            C.HEALER_JOB,
            C.SIMULATION_ABORTED,
            C.STARTED,
            C.ERROR,
            C.INTERRUPTED,
            C.ANY_JOB,
            C.ODB_FILE,
            C.HEADING,
        ],
        callback: SymbolicConstant,
        userData: str = "",
    ):
        """This method specifies a callback function that will be called when the specified message is received
        from the analysis product. For more information, see An example of a callback function.

        Parameters
        ----------
        jobName
            A String specifying the name of the job to be monitored or the SymbolicConstant ANY_JOB.
        messageType
            A SymbolicConstant specifying which message type will call this callback. Possible
            values are:

            - ABORTED
            - ANY_JOB
            - ANY_MESSAGE_TYPE
            - COMPLETED
            - END_STEP
            - ERROR
            - HEADING
            - HEALER_JOB
            - HEALER_TYPE
            - INTERRUPTED
            - ITERATION
            - JOB_ABORTED
            - JOB_COMPLETED
            - JOB_INTERRUPTED
            - JOB_SUBMITTED
            - MONITOR_DATA
            - ODB_FILE
            - ODB_FRAME
            - SIMULATION_ABORTED
            - SIMULATION_COMPLETED
            - SIMULATION_INTERRUPTED
            - SIMULATION_SUBMITTED
            - STARTED
            - STATUS
            - STEP
            - WARNING

        callback
            A Python function to be called. The interface definition of the callback function is
            `def onMessage(jobName, messageType, data, userData)` **jobName** is a
            String. **messageType** is a SymbolicConstant with possible values as listed previously for
            the addMessageCallback method. **data** is a DataObject object. **userData** is the object
            passed as the **userData** argument to the addMessageCallback method.
        userData
            Any Python object or None. This object is passed to the callback function.
        """
        ...

    @abaqus_method_doc
    def removeMessageCallback(
        self,
        jobName: str,
        messageType: Literal[
            C.HEALER_TYPE,
            C.MONITOR_DATA,
            C.JOB_COMPLETED,
            C.JOB_SUBMITTED,
            C.SIMULATION_SUBMITTED,
            C.STEP,
            C.WARNING,
            C.JOB_ABORTED,
            C.SIMULATION_INTERRUPTED,
            C.COMPLETED,
            C.ITERATION,
            C.STATUS,
            C.SIMULATION_COMPLETED,
            C.ODB_FRAME,
            C.ABORTED,
            C.JOB_INTERRUPTED,
            C.ANY_MESSAGE_TYPE,
            C.END_STEP,
            C.HEALER_JOB,
            C.SIMULATION_ABORTED,
            C.STARTED,
            C.ERROR,
            C.INTERRUPTED,
            C.ANY_JOB,
            C.ODB_FILE,
            C.HEADING,
        ],
        callback: str,
        userData: str,
    ):
        """This method removes a callback function. You specify the callback function to remove using the same
        arguments you used to add the callback.

        Parameters
        ----------
        jobName
            A String specifying the name of the job to be monitored or the SymbolicConstant ANY_JOB.
        messageType
            A SymbolicConstant specifying which message type will call this callback. Possible
            values are:
            - ABORTED
            - ANY_JOB
            - ANY_MESSAGE_TYPE
            - COMPLETED
            - END_STEP
            - ERROR
            - HEADING
            - HEALER_JOB
            - HEALER_TYPE
            - INTERRUPTED
            - ITERATION
            - JOB_ABORTED
            - JOB_COMPLETED
            - JOB_INTERRUPTED
            - JOB_SUBMITTED
            - MONITOR_DATA
            - ODB_FILE
            - ODB_FRAME
            - SIMULATION_ABORTED
            - SIMULATION_COMPLETED
            - SIMULATION_INTERRUPTED
            - SIMULATION_SUBMITTED
            - STARTED
            - STATUS
            - STEP
            - WARNING
        callback
            A Python function to be called; it must be the same as the **callback** argument specified
            in the original call to addMessageCallback.
        userData
            Any Python object or None; it must be the same as the **userData** argument specified in
            the original call to addMessageCallback.
        """
        ...

    @abaqus_method_doc
    def checkMonitorStatus(self):
        """This method raises a MonitorError exception if the monitoring status is not ENABLED.

        Raises
        ------
        MonitorError
            Status is not ENABLED
        """
        ...



"""
Mdb commands
============
The following command upgrades a model database (.cae) to the current release and
writes the upgraded model database to a new file.

This page discusses:

* upgradeMdb
* CombineOptResults

"""
from __future__ import annotations

from typing import Sequence, Union

from typing_extensions import Literal

from abqpy.decorators import abaqus_function_doc

from ..UtilityAndView.abaqusConstants import ALL, FIRST, INITIAL_AND_LAST
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .Mdb import Mdb


def upgradeMdb(existingMdbPath: str, upgradedMdbPath: str) -> None:
    """This method upgrades an existing Mdb object to the current release and writes the upgraded version of the
    Mdb object to a file. In addition, Abaqus/CAE writes.

    information about the status of the upgrade to the log file ( upgradedMdbPath.log
    ).

    Parameters
    ----------
    existingMdbPath : str
        A String specifying the path to the file containing the model database to be
        upgraded.
    upgradedMdbPath : str
        A String specifying the path to the file that will contain the upgraded model
        database.

    Raises
    ------
    MdbError
        Cannot convert file
    """
    ...


def CombineOptResults(
    optResultLocation: str,
    optIter: Literal[C.INITIAL_AND_LAST, C.NONE, C.ALL, C.LAST, C.EVERY_NCYCLES, C.SPECIFY] = INITIAL_AND_LAST,
    nValues: Union[int, Sequence[int], Literal[C.ALL]] = ALL,
    models: Union[Sequence[str], Literal[C.ALL]] = ALL,
    steps: Union[Sequence[str], Literal[C.ALL]] = ALL,
    analysisFieldVariables: Union[Sequence[str], Literal[C.ALL]] = ALL,
    includeResultsFrom: Literal[C.ORIGINAL_MODEL, C.FIRST, C.LAST] = FIRST,
    originalModel: str = ...,
) -> None:
    """This method combines the results from existing ODB files for each optimization cycle and writes a merged
    ODB file.

    Parameters
    ----------
    optResultLocation : str
        A String specifying the path to the folder in which optimization results are present.
    optIter
        A Symbolic Constant to specify the optimization cycles from which the results
        should be merged. The possible values are INITIAL_AND_LAST, NONE, ALL, LAST,
        EVERY_NCYCLES, SPECIFY. The default value is INITIAL_AND_LAST.
    nValues
        An Int or a tuple of Ints specifying the optimization cycles from which the
        results should be merged. This argument is used only when EVERY_NCYCLES or
        SPECIFY is selected for optIter. The default value is ALL.
    models
        A tuple of strings specifying the list of models for which the merging of
        results is performed. The default value is ALL.
    steps
        A tuple of strings specifying the list of steps from the selected models to
        be included in the odb merge. The default value is ALL.
    analysisFieldVariables
        A tuple of strings specifying the list of analysisFieldVariables to be
        included in the odb merge. The default value is ALL.
    includeResultsFrom
       A Symbolic Constant to specify the target odb to which the results will be
       merged. The possible values are ORIGINAL_MODEL, FIRST or LAST. The default
       value is FIRST.
    originalModel
        A String to specify the path of target odb if includeResultsFrom is set to
        ORIGINAL_MODEL.
    """
    ...


@abaqus_function_doc
def openMdb(pathName: str) -> Mdb:
    """This method opens an existing model database file.

    .. note::
        This function can be accessed by::

            Mdb

    Parameters
    ----------
    pathName: str
        A String specifying the path to the model database file to open. If you do not provide a
        file extension, Abaqus/CAE attempts to open the file with .cae appended to the path.

    Returns
    -------
    Mdb
        A Mdb object

    Raises
    ------
    MdbError
        invalid model database;
        If the file is an invalid model database
    MdbError
        incompatible release number, expected *<Abaqus release>*, got *<earlier or later Abaqus release>*;
        If the file contains a model database from an Abaqus release other than the Abaqus
        release you are currently running
    MdbError
        cannot open file; may be in use by another CAE session;
        If the model database file is already opened in write mode
    MdbError
        cannot open file;
        If the command fails to open the model database file for reasons not mentioned above
    """
    return Mdb(pathName)



import ast
import os
import sys
import warnings

from .cli import abaqus

ABAQUS_COMMAND_OPTIONS = {"noGUI": True}


def run(cae: bool = True) -> None:
    """Runs Abaqus command in system's CLI.

    This function uses the top level script file to run the Abaqus
    command, and the arguments passed to it

    Parameters
    ----------
    cae : bool, optional
        Whether to use `abaqus cae` command, True for `abaqus cae`, False for `abaqus python`, by default True
    """
    # check if in debug mode and run
    debug = os.environ.get("ABQPY_DEBUG", "false").lower() == "true"
    gettrace = getattr(sys, "gettrace", None)
    debug = debug or (gettrace is not None and gettrace())

    # Check if it is imported by sphinx to generate docs
    make_docs = os.environ.get("ABQPY_MAKE_DOCS", "false").lower() == "true"

    # If in debug mode or making docs, return
    if debug:
        warnings.warn("You are running the script in debug mode, the script will run in the current Python session "
                      "and it will not be submitted to Abaqus.")  # fmt: skip
        return
    elif make_docs:
        return

    # If it is a jupyter notebook, convert it to python script
    try:  # If it is a jupyter notebook
        import ipynbname

        filePath = ipynbname.path()
        print("You are running a jupyter notebook, it will be converted to a pure python script.")
        os.system(f"jupyter nbconvert --to python {filePath}")
        filePath = os.path.relpath(filePath.with_suffix(".py"))
    except (FileNotFoundError, ImportError, Exception):
        # Get the main script file
        main = sys.modules["__main__"]
        if not hasattr(main, "__file__") or main.__file__ is None:
            raise RuntimeError("Cannot find the main script file, please run the script in a file.")
        filePath = os.path.relpath(main.__file__)

    # Alternative to use abaqus command line options at run time
    print("The script will be submitted to Abaqus next and the current Python session will be closed.")
    options: dict = ast.literal_eval(os.environ.get("ABAQUS_COMMAND_OPTIONS", str(ABAQUS_COMMAND_OPTIONS)))
    if cae:
        gui = options.pop("gui", None)
        noGUI = options.pop("noGUI", None)
        options["gui"] = gui if gui is not None else not noGUI if noGUI is not None else False
        abaqus.cae(filePath, *sys.argv[1:], **options)
    else:
        abaqus.python(filePath, *sys.argv[1:], **options)
    sys.exit(0)



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc

from ..UtilityAndView.abaqusConstants import OFF, Boolean, SymbolicConstant
from .LoadState import LoadState


@abaqus_class_doc
class InertiaReliefState(LoadState):
    """The InertiaReliefState object stores the propagating data for an inertia relief load in a step. One
    instance of this object is created internally by the InertiaRelief object for each step. The instance is
    also deleted internally by the InertiaRelief object. The InertiaReliefState object has no constructor or
    methods. The InertiaReliefState object is derived from the LoadState object.

    .. note::
        This object can be accessed by::

            import load
            mdb.models[name].steps[name].loadStates[name]

        The corresponding analysis keywords are:

        - INERTIA RELIEF
    """

    #: A Boolean specifying the 1-direction as a free direction.
    u1: Boolean = OFF

    #: A Boolean specifying the 2-direction as a free direction.
    u2: Boolean = OFF

    #: A Boolean specifying the 3-direction as a free direction.
    u3: Boolean = OFF

    #: A Boolean specifying the rotation about the 1-direction as a free direction.
    ur1: Boolean = OFF

    #: A Boolean specifying the rotation about the 2-direction as a free direction.
    ur2: Boolean = OFF

    #: A Boolean specifying the rotation about the 3-direction as a free direction.
    ur3: Boolean = OFF

    #: A SymbolicConstant specifying the propagation state of the Boolean that identifies the
    #: local 1-direction as a free direction. Possible values are UNSET, SET, UNCHANGED, and
    #: MODIFIED.
    u1State: SymbolicConstant

    #: A SymbolicConstant specifying the propagation state of the Boolean that identifies the
    #: local 2-direction as a free direction. Possible values are UNSET, SET, UNCHANGED, and
    #: MODIFIED.
    u2State: SymbolicConstant

    #: A SymbolicConstant specifying the propagation state of the Boolean that identifies the
    #: local the 3-direction as a free direction. Possible values are UNSET, SET, UNCHANGED,
    #: and MODIFIED.
    u3State: SymbolicConstant

    #: A SymbolicConstant specifying the propagation state of the Boolean that identifies
    #: rotation about the local 1-direction as a free direction. Possible values are UNSET,
    #: SET, UNCHANGED, and MODIFIED.
    ur1State: SymbolicConstant

    #: A SymbolicConstant specifying the propagation state of the Boolean that identifies the
    #: rotation about the local the 2-direction as a free direction. Possible values are UNSET,
    #: SET, UNCHANGED, and MODIFIED.
    ur2State: SymbolicConstant

    #: A SymbolicConstant specifying the propagation state of the Boolean that identifies the
    #: rotation about the local the 3-direction as a free direction. Possible values are UNSET,
    #: SET, UNCHANGED, and MODIFIED.
    ur3State: SymbolicConstant

    #: A Boolean specifying whether the inertia relief loading should remain fixed at the
    #: current loading at the start of the step. The default value is OFF.
    fixed: Boolean = OFF

    #: A SymbolicConstant specifying the propagation state of the Boolean that identifies
    #: whether the inertia relief load should remain fixed at current level at the start of the
    #: step. Possible values are UNSET, SET, UNCHANGED, and MODIFIED.
    fixedState: SymbolicConstant

    #: A SymbolicConstant specifying the propagation state of the reference point of the
    #: inertia relief load. Possible values are UNSET, SET, UNCHANGED, and MODIFIED.
    referencePointState: SymbolicConstant

    #: A tuple of Floats specifying the point about which rotations are defined. The point can
    #: be specified only for certain combinations of free directions. The **referencePoint**
    #: argument can be one of the following:
    #: - The **X**, **Y** and **Z** coordinates of a fixed rotation point.
    #: - A point on the rotation axis.
    #: - A point on the symmetry line.
    referencePoint: float | None = None

    #: A SymbolicConstant specifying the propagation state of the **amplitude** member. Possible
    #: values are UNSET, SET, UNCHANGED, and FREED.
    amplitudeState: SymbolicConstant

    #: A SymbolicConstant specifying the propagation state of the LoadState object. Possible
    #: values are:
    #:
    #: - NOT_YET_ACTIVE
    #: - CREATED
    #: - PROPAGATED
    #: - MODIFIED
    #: - DEACTIVATED
    #: - NO_LONGER_ACTIVE
    #: - TYPE_NOT_APPLICABLE
    #: - INSTANCE_NOT_APPLICABLE
    #: - BUILT_INTO_BASE_STATE
    status: SymbolicConstant

    #: A String specifying the name of the amplitude reference. The String is empty if the load
    #: has no amplitude reference.
    amplitude: str = ""



from __future__ import annotations

from typing import Sequence

from typing_extensions import Literal

from abqpy.decorators import abaqus_function_doc

from ..UtilityAndView.abaqusConstants import SymbolicConstant
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .Material import Material

""" This command evaluates the behavior of a hyperelastic material under standard test 
conditions. 


.. note::
    This object can be accessed by::
    
        from material import evaluateMaterial

"""


@abaqus_function_doc
def evaluateMaterial(
    material: Material,
    simulationName: str,
    dataSource: Sequence[Literal[C.TEST_DATA, C.COEFFICIENTS]],
    strainEnergyPotentials: Sequence[Literal[C.POLY_N1]],
    marlowData: SymbolicConstant | None = None,
    marlowDataType: SymbolicConstant | None = None,
    testDataTypes: SymbolicConstant | None = None,
    uniaxialStrainRange: float | None = None,
    biaxialStrainRange: float | None = None,
    planarStrainRange: float | None = None,
    volumeRatioRange: float | None = None,
    simpleShearStrainRange: float | None = None,
    viscoDataSource: SymbolicConstant | None = None,
    viscoTestDataTypes: SymbolicConstant | None = None,
    relaxationTime: float | None = None,
    creepTime: float | None = None,
):
    """This method evaluates the behavior of a hyperelastic material under standard test conditions.

    .. note::
        This function can be accessed by::

            evaluateMaterial

    Parameters
    ----------
    material
        A Material object.
    simulationName
        A String specifying the name to be used for the material evaluation simulation.
    dataSource
        A SymbolicConstant specifying whether test data or coefficients should be used for the
        material definition in the unit element tests. Possible values are TEST_DATA or
        COEFFICIENTS.
    strainEnergyPotentials
        A sequence of SymbolicConstants specifying for which material models the material is to
        be evaluated. Possible values are

        - POLY_N1
        - POLY_N2
        - POLY_N3
        - POLY_N4
        - POLY_N5
        - POLY_N6,
        - OGDEN_N1
        - OGDEN_N2
        - OGDEN_N3
        - OGDEN_N4
        - OGDEN_N5
        - OGDEN_N6
        - REDUCED_POLY_N1,
        - REDUCED_POLY_N2
        - REDUCED_POLY_N3
        - REDUCED_POLY_N4
        - REDUCED_POLY_N5
        - REDUCED_POLY_N6,
        - ARRUDA_BOYCE
        - VAN_DER_WAALS
        - YEOH
        - MOONEY_RIVLIN
        - NEO_HOOKE.

        **Note**: The options
        POLY_N3, POLY_N4, POLY_N5, and POLY_N6 are valid only if the material was defined by
        providing coefficients of the strain energy potential.
    marlowData
        None or a sequence of SymbolicConstants specifying the types of test data to be included
        in the material definition of the Marlow material that is being evaluated. Possible
        values are UNIAXIAL, BIAXIAL, PLANAR, or VOLUMETRIC. The default value is None.
    marlowDataType
        None or a SymbolicConstant specifying the input data type for the Marlow material model.
        Possible values are TENSION, COMPRESSION, or BOTH.
    testDataTypes
        A sequence of SymbolicConstants specifying the types of test data to be included in the
        material definition of the material being evaluated. Possible values are UNIAXIAL,
        BIAXIAL, PLANAR, and VOLUMETRIC.
    uniaxialStrainRange
        A tuple of Floats specifying minimum and maximum nominal strains to be applied in the
        uniaxial tension test.
    biaxialStrainRange
        A tuple of Floats specifying the minimum and maximum nominal strains to be applied in
        the biaxial tension test.
    planarStrainRange
        A tuple of Floats specifying the minimum and maximum nominal strains to be applied in
        the planar test. The planar test is equivalent to a pure shear test.
    volumeRatioRange
        A tuple of Floats specifying the minimum and maximum compressive volume ratio.
    simpleShearStrainRange
        A tuple of Floats specifying the minimum and maximum nominal strains to be applied in
        the simple shear test.
    viscoDataSource
        None or a SymbolicConstant specifying whether test data or coefficients should be used
        for the viscoelastic material definition in the element tests. Possible values are
        TEST_DATA or COEFFICIENTS. The default value is None.
    viscoTestDataTypes
        None or a sequence of SymbolicConstants specifying the types of test data to be included
        in the material definition of the viscoelastic material being evaluated. Possible values
        are UNIAXIAL, BIAXIAL, PLANAR, or VOLUMETRIC. The default value is None.
    relaxationTime
        None or a Float specifying the time period for the stress relaxation response mode. The
        default value is None.
    creepTime
        None or a Float specifying the time period for the creep response mode. The default
        value is None.

    Raises
    ------
    MaterialEvaluationError: POLY_N3, POLY_N4, POLY_N5, or POLY_N6 not allowed for **dataSource** = TEST_DATA
        If **dataSource** = TEST_DATA and **strainEnergyPotentials** contains POLY_N3, POLY_N4,
        POLY_N5, or POLY_N6.
    MaterialEvaluationError: material evaluation failed, see **path to data file**
        If the material evaluation failed.
    MaterialEvaluationError: Material evaluation is currentlysupported only for hyperelastic materials
        If the material type of the material to be evaluated is not hyperelastic.
    """
    ...



from __future__ import annotations

from typing import Sequence

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ...UtilityAndView.abaqusConstants import OFF, Boolean
from .ConstrainedSketchGeometry import ConstrainedSketchGeometry


@abaqus_class_doc
class getPointAtDistance(ConstrainedSketchGeometry):
    @abaqus_method_doc
    def __init__(self, point: Sequence[float], distance: str, percentage: Boolean = OFF):
        """This method returns a point offset along the given ConstrainedSketchGeometry from the given end by a
        specified arc length distance or a percentage of the total length of the ConstrainedSketchGeometry
        object.

        Parameters
        ----------
        point
            A pair of Floats specifying the point from which the distance is to be measured.
        distance
            A float specifying the arc length distance along the ConstrainedSketchGeometry from the
            **point** at which the required point is situated.
        percentage
            A Boolean that specifies if the **distance** is an absolute distance or is a fraction
            relative to the length of the ConstrainedSketchGeometry object.

        Returns
        -------
        Sequence[float]
            A pair of floats representing the point along the edge.
        """
        ...



from __future__ import annotations

from typing import Union

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import (
    DAMPING_COEFFICIENT,
    DEFAULT,
    STEP,
    SymbolicConstant,
)
from ..UtilityAndView.abaqusConstants import abaqusConstants as C


@abaqus_class_doc
class ContactDamping:
    """The ContactDamping object specifies damping for a contact interaction property.

    .. note::
        This object can be accessed by::

            import interaction
            mdb.models[name].interactionProperties[name].damping

        The table data for this object are:

        - If **definition** = DAMPING_COEFFICIENT and **clearanceDependence** = STEP, the table data specify the following:

            - Damping coefficient.

        - If **definition** = DAMPING_COEFFICIENT and **clearanceDependence** = LINEAR or BILINEAR, the table data specify the following:

            - Damping coefficient.
            - Clearance.

            Two pairs must be given for **clearanceDependence** = LINEAR and three pairs for **clearanceDependence** = BILINEAR. The first pair must have **clearance** = 0.0, and the last pair must have **coefficient** = 0.0.
        - If **definition** = CRITICAL_DAMPING_FRACTION, the table data specify the following:

            - Critical damping fraction.

        The corresponding analysis keywords are:

        - CONTACT DAMPING
    """

    #: A SymbolicConstant specifying the method used to define the damping. Possible values are
    #: DAMPING_COEFFICIENT and CRITICAL_DAMPING_FRACTION. The default value is
    #: DAMPING_COEFFICIENT.
    definition: SymbolicConstant = DAMPING_COEFFICIENT

    #: The SymbolicConstant DEFAULT or a Float specifying the tangential damping coefficient
    #: divided by the normal damping coefficient. The default value is DEFAULT.
    tangentFraction: Union[SymbolicConstant, float] = DEFAULT

    #: A SymbolicConstant specifying the variation of the damping coefficient or fraction with
    #: respect to clearance. Possible values are STEP, LINEAR, and BILINEAR. The default value
    #: is STEP.If **definition** = CRITICAL_DAMPING_FRACTION, the only possible value is STEP.
    clearanceDependence: SymbolicConstant = STEP

    #: A tuple of pairs of Floats specifying the damping properties. The items in the table
    #: data are described below.
    table: float | None = None

    @abaqus_method_doc
    def __init__(
        self,
        definition: Literal[C.DAMPING_COEFFICIENT, C.CRITICAL_DAMPING_FRACTION] = DAMPING_COEFFICIENT,
        tangentFraction: Union[Literal[C.DEFAULT], float] = DEFAULT,
        clearanceDependence: Literal[C.STEP, C.CRITICAL_DAMPING_FRACTION, C.BILINEAR, C.LINEAR] = STEP,
        table: tuple = (),
    ):
        """This method creates a ContactDamping object.

        .. note::
            This function can be accessed by::

                mdb.models[name].interactionProperties[name].Damping

        Parameters
        ----------
        definition
            A SymbolicConstant specifying the method used to define the damping. Possible values are
            DAMPING_COEFFICIENT and CRITICAL_DAMPING_FRACTION. The default value is
            DAMPING_COEFFICIENT.
        tangentFraction
            The SymbolicConstant DEFAULT or a Float specifying the tangential damping coefficient
            divided by the normal damping coefficient. The default value is DEFAULT.
        clearanceDependence
            A SymbolicConstant specifying the variation of the damping coefficient or fraction with
            respect to clearance. Possible values are STEP, LINEAR, and BILINEAR. The default value
            is STEP.If **definition** = CRITICAL_DAMPING_FRACTION, the only possible value is STEP.
        table
            A sequence of pairs of Floats specifying the damping properties. The items in the table
            data are described below.

        Returns
        -------
        ContactDamping
            A ContactDamping object.
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the ContactDamping object."""
        ...



from __future__ import annotations

from typing import List

from .RayleighDampingByFrequencyComponent import RayleighDampingByFrequencyComponent

RayleighDampingByFrequencyComponentArray = List[RayleighDampingByFrequencyComponent]



from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ....UtilityAndView.abaqusConstants import ISOTROPIC, OFF, Boolean
from ....UtilityAndView.abaqusConstants import abaqusConstants as C
from .SaturationDependence import SaturationDependence
from .VelocityDependence import VelocityDependence


@abaqus_class_doc
class Permeability:
    """The Permeability object defines permeability for pore fluid flow.

    .. note::
        This object can be accessed by::

            import material
            mdb.models[name].materials[name].permeability
            import odbMaterial
            session.odbs[name].materials[name].permeability

        The table data for this object are:

        - If **type** = ISOTROPIC, the table data specify the following:

            - :math:`k`.
            - Void ratio, :math:`e`.
            - Temperature, if the data depend on temperature.
        - If **type** = ORTHOTROPIC, the table data specify the following:

            - :math:`k_{11}`.
            - :math:`k_{22}`.
            - :math:`k_{33}`.
            - Void ratio, :math:`e`.
            - Temperature, if the data depend on temperature.
        - If **type** = ANISOTROPIC, the table data specify the following:

            - :math:`k_{11}`.
            - :math:`k_{12}`.
            - :math:`k_{22}`.
            - :math:`k_{13}`.
            - :math:`k_{23}`.
            - :math:`k_{33}`.
            - Void ratio, :math:`e`.
            - Temperature, if the data depend on temperature.

        The corresponding analysis keywords are:

        - PERMEABILITY
    """

    #: A SaturationDependence object specifying the dependence of the permeability of a
    #: material on the saturation of the wetting liquid.
    saturationDependence: SaturationDependence = SaturationDependence(((),))

    #: A VelocityDependence object specifying the dependence of the permeability of a material
    #: on the velocity of fluid flow.
    velocityDependence: VelocityDependence = VelocityDependence(((),))

    @abaqus_method_doc
    def __init__(
        self,
        specificWeight: float,
        inertialDragCoefficient: float,
        table: tuple,
        type: Literal[C.ANISOTROPIC, C.ISOTROPIC, C.ORTHOTROPIC] = ISOTROPIC,
        temperatureDependency: Boolean = OFF,
        dependencies: int = 0,
    ):
        r"""This method creates a Permeability object.

        .. note::
            This function can be accessed by::

                mdb.models[name].materials[name].Permeability
                session.odbs[name].materials[name].Permeability

        Parameters
        ----------
        specificWeight
            A Float specifying the specific weight of the wetting liquid, :math:`\gamma_w`.
        inertialDragCoefficient
            A Float specifying The inertial drag coefficient of the wetting liquid, :math:`\gamma_w`.
        table
            A sequence of sequences of Floats specifying the items described below.
        type
            A SymbolicConstant specifying the type of permeability. Possible values are ISOTROPIC,
            ORTHOTROPIC, and ANISOTROPIC. The default value is ISOTROPIC.
        temperatureDependency
            A Boolean specifying whether the data depend on temperature. The default value is OFF.
        dependencies
            An Int specifying the number of field variable dependencies. The default value is 0.

        Returns
        -------
        Permeability
            A Permeability object.

        Raises
        ------
        RangeError
        """
        ...

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the Permeability object.

        Raises
        ------
        RangeError
        """
        ...
